@@ -1,6 +1,7 @@
 /* libwpd
  * Copyright (C) 2003 William Lachance (william.lachance@sympatico.ca)
  * Copyright (C) 2003-2004 Marc Maurer (j.m.maurer@student.utwente.nl)
+ * Copyright (C) 2005 Fridrich Strba (fridrich.strba@bluewin.ch)
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -40,46 +41,41 @@ WP42HLListener::WP42HLListener(std::vector<WPXPageSpan *> *pageList, WPXHLListen
 
 void WP42HLListener::insertCharacter(const uint16_t character)
 {
+	if (m_ps->m_isSpanOpened)
+		_openSpan();
 	appendUCS4(m_textBuffer, (uint32_t)character);
 }
 
 void WP42HLListener::insertTab(const uint8_t tabType, const float tabPosition)
 {
-	_flushText();
-	m_listenerImpl->insertTab();
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isSpanOpened)
+			_openSpan();
+		else
+			_flushText();
+
+		m_listenerImpl->insertTab();
+	}
 }
 
 void WP42HLListener::insertEOL()
 {
-	_flushText();
-	m_ps->m_numDeferredParagraphBreaks++;
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+			_openSpan();
+		if (m_ps->m_isParagraphOpened)
+			_closeParagraph();
+		if (m_ps->m_isListElementOpened)
+			_closeListElement();
+	}
 }
 
 void WP42HLListener::endDocument()
 {
-	// corner case: document ends in a list element
-	/*if (m_parseState->m_styleStateSequence.getCurrentState() != NORMAL)
-	{
-		_flushText(); // flush the list text
-		m_parseState->m_styleStateSequence.setCurrentState(NORMAL);
-		_flushText(true); // flush the list exterior (forcing a line break, to make _flushText think we've exited a list)
-	}*/
-	// corner case: document contains no end of lines
-	/*else*/ if (!m_ps->m_isParagraphOpened && !m_ps->m_isParagraphClosed)
-	{
-		_flushText();
-	}
-	// NORMAL(ish) case document ends either inside a paragraph or outside of one,
-	// but not inside an object
-	else if (!m_ps->m_isParagraphClosed || !m_ps->m_isParagraphOpened)
-	{
-		_flushText();
-	}
-
-	// the only other possibility is a logical contradiction: a paragraph
-	// may not be opened and closed at the same time
-
-	// close the document nice and tight
+	_closeSpan();
+	_closeParagraph();
 	_closeSection();
 	_closePageSpan();
 	m_listenerImpl->endDocument();
@@ -92,9 +88,7 @@ void WP42HLListener::endDocument()
 
 void WP42HLListener::attributeChange(const bool isOn, const uint8_t attribute)
 {
-
-	// flush everything which came before this change
-	_flushText();
+	_closeSpan();
 
 	uint32_t textAttributeBit = 0;
 
@@ -137,55 +131,15 @@ void WP42HLListener::attributeChange(const bool isOn, const uint8_t attribute)
 		m_ps->m_textAttributeBits |= textAttributeBit;
 	else
 		m_ps->m_textAttributeBits ^= textAttributeBit;
-
-	m_ps->m_textAttributesChanged = true;
 }
 
 /****************************************
  private functions
 *****************************************/
 
-/*
-TODO: this function needs to be fleshed out
-*/
-void WP42HLListener::_flushText(const bool fakeText)
+void WP42HLListener::_flushText()
 {
-	// create a new section, and a new paragraph, if our section attributes have changed and we have inserted
-	// something into the document (or we have forced a break, which assumes the same condition)
-	if (m_ps->m_sectionAttributesChanged && (m_textBuffer.len() > 0 || m_ps->m_numDeferredParagraphBreaks > 0/* || fakeText*/))
-	{
-		_openSection();
-		//if (fakeText)
-			_openParagraph();
-	}
-
-	if (m_ps->m_numDeferredParagraphBreaks > 0)
-	{
-		if (!m_ps->m_isParagraphOpened //&&
-			// !(m_parseState->m_isTableOpened && !m_parseState->m_isTableCellOpened) // don't allow paragraphs to be opened when we have already opened a table, but no cell yet. - MARCM (is it really correct, or should this be fixed elsewhere??)
-		)
-			m_ps->m_numDeferredParagraphBreaks++;
-
-		while (m_ps->m_numDeferredParagraphBreaks > 1)
-			_openParagraph();
-		_closeParagraph();
-		m_ps->m_numDeferredParagraphBreaks = 0; // compensate for this by requiring a paragraph to be opened
-	}
-	else if (m_ps->m_textAttributesChanged && m_textBuffer.len())
-	{
-		_openSpan();
-		m_ps->m_textAttributesChanged = false;
-	}
-
 	if (m_textBuffer.len())
-	{
-		if (!m_ps->m_isParagraphOpened)
-		{
-			_openParagraph();
-			_openSpan();
-		}
-
 		m_listenerImpl->insertText(m_textBuffer);
-		m_textBuffer.clear();
-	}
+	m_textBuffer.clear();
 }
