@@ -77,8 +77,8 @@ protected:
 	virtual void _handleSubDocument(uint16_t textPID, const bool isHeaderFooter, WPXTableList tableList, int nextTableIndice = 0) {}
 
 private:
-	void _flushText(const bool fakeText=false);
-	void _flushList() {};
+	void _flushText();
+	void _changeList() {};
 
 	WPXString m_textBuffer;
 
